import matplotlib
matplotlib.use("Agg")
import matplotlib.pyplot as plt
from fa2l import force_atlas2_layout
from adjustText import adjust_text
import networkx as nx

from .utils import set_node_size, set_node_color, set_node_label
from .utils import edgecolor_by_source, filter_graph, get_subgraph_pos
from .utils import draw_networkx_nodes_custom
from .scaling import get_auto_scale, scale_layout


def draw_forceatlas2_network(G,
                             pos=None,
                             fa2l_iterations=50, fa2l_scaling_ratio=38,
                             scale="auto",
                             num_labels=None,
                             node_list=None,
                             node_color='red', color_by=None, colormap=None,
                             node_size=10,
                             size_field=None, min_size=0.1, max_size=100,
                             with_labels=False, label_field=None,
                             filter_by=None, top=None,
                             adjust_labels=True,
                             node_opacity=1, edge_opacity=0.05,
                             font_size=8, font_color='k', font_family='sans-serif',
                             filename="untitled.png", title=None,
                             edge_color="lightgray",
                             edge_color_by_source=False,
                             figsize=(10, 10), fig_dpi=100, **kwargs):
    """ Main function for drawing graph.

    The function has sensible defaults. If no scale is provided scale for the graph is set automatically. 
    If no pos is given then force atlas 2 layout is used. Other parameters can be described below can be 
    used to customize different aspects of the visualization. 

    See Notes section for dependencies. 

    Check README for examples.


    Parameters
    ----------
    G : nx.Graph

        A networkx graph.

    pos : dict or None. optional. default None.

        pos containing positions for the graph generated by some network layout algorithm. 

        If None, force_atlas2_layout function from fa2l package is used with default parameters to generate 
        the layout for the graph. To use with user given parameters for the force_atlas2_layout, calculate
        pos outside draw function and then pass to the draw function.

        Pos can also be calculated with  any other network layout algorithms like nx.spring_layout(G) 
        and passed to the draw function. 

    fa2l_iterations : int, optional. Default 50..

        number of times to run the default force atlas 2 layout algorithm from fa2l package.

    fa2l_scaling_ratio : float, optional. Default 38.

        How much repulsion you want. More makes a more sparse graph.

    scale : "auto" or float or int. optional. default "auto".

        Used for scaling the graph automatically.Also used for expanding and contracting the graph.

        If "auto", then scale is set automatically. see : mediaviz.scaling.get_auto_scale(...) for how
        scale is set. If given a float or int, positions are scaled according to the number. 

        For example, scale = 2 will expand the graph to twice of its current size while scale = 0.5 will 
        contract the graph to half of its original size. 

        When scaling graph node sizes are not changed, only the relative positions are changed. 

        For example, after doubling a position (2,4) will become (4,8).

    num_labels : int, optional. default None.

        Number of nodes to label sorted by the node size. 

        Recommended to use only with varying node sizes.

        If all node sizes are equal it will just return the nodes from G.nodes() in the same order.

    node_list : list. optional, default None.
        list of nodes to draw. if None and there's no filtering done all nodes are drawn.

    node_color : str or list. default 'red'.

        Multiple variations are allowed. 

        node_color can be color name like "r","b" or hex code in string format.

        node_color can also be a list of colors like ["r","b"...] or ["#FFFFFF","#..."...]. 

        If node_colors are being passed then color_by and colormap should be None(their defaults)

    color_by : node attribute. optional. Default None.

        categorical node attribute to color the nodes by. 

        If color by is given, a colormap dictionary must also be provided.

        Example : if "gender" in graph node attribute, then color_by = "gender" and 
        colormap = {"M":'b',"F":'r'}

    colormap : dict, optional. Default None.

        dictionaries containing color assignment for each value of the categorical node attribute in color_by.

        Assumes color_by is a categorical node_attribute like "gender" or "partition" or "country". 

        colormap is then a dictionary that assigns color for each unique value of that categorical node 
        attribute. 

        Example : If there's two unique values "M" and "F" in "gender" node attribute, then 
        color_by = "gender" and colormap = {"M":'b',"F":'r'}.

    node_size : int or float or list. Default 10.

        Multiple values are supported.
        
        int or float value can be provided to make all node sizes same.

        list of node sizes can also be passed.

        if node_size is given, size_field should not be used. Default of size_field is None.

    size_field : str, optional. Default None.

        Node attribute to resize the nodes by. Must be numeric node attribute.

        If given node sizes are resized to [min_size, max_size] using set_node_size from utils module.

    min_size : float, optional. Default 0.1

        Minimum size for the nodes.

    max_size : float , optional. Default : 100.

        maximum size for the nodes

    with_labels: bool, optional. Default False.

        Whether to show the labels or not. If label_field is not provided node names are used.

    filter_by : str, optional. Default None.
         
         Numeric Node attribute to filter the graph by. If filter_by is given, top must be provided. 

        Filter_by filters the graph to draw only the top k nodes. 

        For example, if filter_by = "inlink_count" and top = 100, the graph is filtered to
        the subgraph containing the top 100 nodes sorted by inlink count and only the subgraph 
        is drawn.

    top : int, optional. Default None.

        Number of largest nodes to keep after filtering using filter_by.

    label_field : str, optional. Default None.

        Node attribute to label the nodes by.

    adjust_labels : bool, optional. Default True.

        If True, Adjust Text package is used for adjusting the labels to prevent label overlap.

        Label texts are iteratively adjusted until there's no overlap.

        See : https://github.com/Phlya/adjustText for details. 

    node_opacity : float. Default 1

        Node alpha.
    
    edge_opacity : float.Default 0.05.

        Edge alpha.

    font_size : int, optional. Default 8.
        
        Label font size.

    font_color : str, optional. Default 'k'

        Label font color. 

    font_family : str. optional. Default 'sans-serif'

        Label font family.

    filename : str, optional. Default "untitled.png"

        File name to save the figure by. 
        See fname from https://matplotlib.org/api/_as_gen/matplotlib.pyplot.savefig.html

    title : str, optional. Default None.

        Plt title.

    edge_color_by_source : bool, optional. Default False.

        If true, edge colors are same as the source node for the edge. If edge_color is given, then 
        edge_color_by_source should be set to default False.

    figsize : tuple, optional. Default : (10,10)
        figure size.

    fig_dpi : float, optional. Default : 100. 
        See https://matplotlib.org/api/figure_api.html for details.


    """
    if node_list:
        G = nx.subgraph(node_list)

    if type(G) == nx.DiGraph:
        G = max(nx.weakly_connected_component_subgraphs(
            G), key=len).to_undirected()

    if pos is None:
        pos = force_atlas2_layout(
            G,
            iterations=fa2l_iterations,
            pos_list=None,
            node_masses=None,
            outbound_attraction_distribution=False,
            lin_log_mode=False,
            prevent_overlapping=False,
            edge_weight_influence=1.0,
            jitter_tolerance=1.0,
            barnes_hut_optimize=True,
            barnes_hut_theta=1.0,
            scaling_ratio=fa2l_scaling_ratio,
            strong_gravity_mode=False,
            multithread=False,
            gravity=1.0)

    if scale == "auto":
        if size_field:
            original_node_sizes = dict(zip(G.nodes(), set_node_size(
                G, size_field=size_field, min_size=min_size, max_size=max_size)))
        elif type(node_size) == int or type(node_size) == float:
            original_node_sizes = dict(
                zip(G.nodes(), [node_size]*len(G.nodes())))
        else:
            original_node_sizes = dict(zip(G.nodes(), node_size))
        scale = get_auto_scale(G, pos, original_node_sizes, k=20)
        print("scale is " + str(scale))
        pos = scale_layout(pos, scale)
    elif scale:
        pos = scale_layout(pos, scale)

    if with_labels is True and num_labels is None:
        num_labels = len(G.nodes())

    if filter_by:
        G = filter_graph(G, filter_by=filter_by, top=top)
        pos = get_subgraph_pos(G, pos)

    if color_by:
        node_color = set_node_color(G, color_by=color_by, colormap=colormap)

    if size_field:
        node_size = set_node_size(
            G, size_field=size_field, min_size=min_size, max_size=max_size)
    elif type(node_size) == int or type(node_size) == float:
        node_size = [node_size]*len(G.nodes())

    if edge_color_by_source:
        edge_color = edgecolor_by_source(G, node_color)

    if with_labels and label_field:
        node_labels = set_node_label(G, label_field=label_field)
        subset_label_nodes = sorted(zip(G.nodes(), node_size),
                                    key=lambda x: x[1],
                                    reverse=True)[0:num_labels]
        subset_labels = {n[0]: node_labels[n[0]] for n in subset_label_nodes}

    if with_labels and label_field is None:
        subset_labels = dict((n, n) for n in G.nodes())

    # plot the visualization

    fig = plt.figure(figsize=figsize, dpi=fig_dpi, **kwargs)
    ax = fig.add_subplot(111)

    # Draw the nodes, edges, labels separately

    draw_networkx_nodes_custom(G,
                               pos=pos, node_size=node_size,
                               node_color=node_color,
                               ax=ax,
                               alpha=node_opacity,
                               **kwargs)
    plt.axis("scaled")
    nx.draw_networkx_edges(
        G, pos=pos, edge_color=edge_color, alpha=edge_opacity, **kwargs)

    if with_labels:
        labels = nx.draw_networkx_labels(
            G, pos=pos, labels=subset_labels, font_size=font_size,
            font_color=font_color, font_family=font_family, **kwargs)
        if adjust_labels:
            # Adjust label overlapping
            x_pos = [v[0] for k, v in pos.items()]
            y_pos = [v[1] for k, v in pos.items()]
            adjust_text(
                texts=list(labels.values()),
                x=x_pos,
                y=y_pos)

    # add title
    if title:
        plt.title(title)

    ax.axis("off")
    # save the plot

    plt.savefig(filename)

    # Show the plot
    plt.show()



import random
import networkx as nx
import community

def _generate_random_hex_color():
    """ Generates handom hex color code and returns them as string
    
    Helper function for setting community colormaps. Does not take any input.
    
    Returns
    -------
    str
        hex code. Example : "#FFFFFF". 

    """
    r = lambda: random.randint(0,255)
    return '#%02X%02X%02X' % (r(),r(),r())


def get_community_colormap(partition):
    """Takes dict of partitions after community detection and returns dict with colormap for partitions.

    The function is currently used with python-louvain package, but it's a general purpose function
    for randomly generating colorlaps from node attributes.Expected to change into 
    get_random_colormap(node_attributes) in the next version.

    Parameters
    ----------
    partition : dict
        Dictionary of form {node1:partition,node2:partition....} generated after community detection.
    
    Examples
    --------
    >>> import community #python-louvain package
    >>> import networkx as nx
    >>> from mediaviz.community_utils import get_community_colormap
    >>> G = nx.florentine_families_graph()
    >>> partitions = community.best_partition(G)
    >>> get_community_colormap(partitions)
    {0: '#A0FD29', 1: '#C3E994', 2: '#61EDD2', 3: '#DCB0DF'}



    Returns
    -------
    dict
        Dictionary containing partitions and colormaps. 

    """
    partitions = set(partition.values())
    result = {p:_generate_random_hex_color() for p in partitions}
    return result

def get_community_graph(G):
    """ Takes a Graph, generates partitions from python-louvain package, sets the partitions as node
    attributes and returns the Graph and partitions.

    Helper function to integrate community detection with the draw function. In the draw function color_by
    parameter takes a node attribute along with a colormap for the attributes. So here python-louvain is used
    for getting the community partitions first and then we set those partitions as node attributes. 

    Examples 
    --------
    >>> import networkx as nx
    >>> G = nx.florentine_families_graph()
    >>> from mediaviz.community_utils import get_community_graph
    >>> G, partitions = get_community_graph(G)

    Parameters
    ----------
    G : nx.Graph
        A networkx graph.
    
    Notes 
    -----

    If G is a digraph , as python-louvain package does not currently support digraphs, it's changed to the 
    undirected graph of the largest weakly connected component.See source if necessary.

    Returns
    -------
    tuple 
        Returns a tuple of form (G, partitions) where G has partitions set as node attributes and 
        partitions are partitions generated from the python-louvain package. 

    """
    if type(G) == nx.DiGraph:
        G = max(nx.weakly_connected_component_subgraphs(G), key=len).to_undirected()
    partitions = community.best_partition(G)
    for node in G.nodes():
        G.node[node]["partition"] = partitions[node]
    return G, partitions
    


def _rotate(point, angle, origin = (0,0),unit = 'degree'):
    """Rotate a point counterclockwise by a given angle around a given origin.

    Angle can be both in radian or degree. Helper function for rotating a layout.

    Parameters
    ----------
    point : tuple 
        position in (x,y) form
    angle : float
        angle to rotate the point
    origin : tuple in (x,y) form
        point will rotate with respect to the origin.
    unit : 'degree'/'radian' to indicate if the angle is in degrees or radians. 
        if given in degrees angle is converted to radians.


    Returns
    -------
    tuple
        rotated point as (x,y) tuple.
    """
    import math
    ox, oy = origin
    px, py = point
    if unit == 'degree':
        angle = math.radians(angle)
    if unit == 'radian':
        angle = angle
    qx = ox + math.cos(angle) * (px - ox) - math.sin(angle) * (py - oy)
    qy = oy + math.sin(angle) * (px - ox) + math.cos(angle) * (py - oy)
    return qx, qy

# rotation example  : pos2 = {k:rotate(v,45,(0.5,0.5)) for k,v in pos2.items()}

def rotation_layout(pos,angle,origin=(0,0),unit="degree"):
    
    """ Rotates the pos to the given angle with respect to origin.
    
    Parameters  
    ----------
    
    pos : dict
        A dictionary with nodes as keys and positions as values.
    
    angle : float
        angle in degree or radian
    
    origin : tuple
        the point will rotate with respect to the origin.
    unit : str
        'degree'/'radian' to indicate if the angle is in degrees or radians.If given in degrees 
        angle is converted to radians.

    Returns
    -------
    dict
        Returns the pos dict with positions rotated by the given angle with respect to the given origin.
    
    """
    return { k:_rotate(v,angle,origin,unit) for k,v in pos.items()}


import math
from itertools import combinations
from fa2l import force_atlas2_layout
import networkx as nx


def _get_distance(pos1, pos2):
    """ Returns distance between two points

    Params : 

    pos1 : tuple in (x,y) format
    pos2 : tuple in (x,y) format

    """
    return math.sqrt((pos2[0]-pos1[0])**2 + (pos2[1]-pos1[1])**2)


def _get_pairwise_distance_between_largest_nodes(G, pos, node_sizes, k=20):
    """ 
    Boolean. Returns the distances between top k largest nodes

    Parameters  
    ----------

    G : A graph object
    pos : dict containing the position of the nodes
    k : number of nodes to consider

    Returns 
    -------

    Returns the dictionary containing the distance between the top k largest nodes.  
    """
    top_k_nodes = [n[0] for n in sorted(
        node_sizes.items(), key=lambda x:x[1], reverse=True)[0:k]]
    distances = {}
    for n1, n2 in combinations(top_k_nodes, 2):
        distances[(n1, n2)] = _get_distance(pos[n1], pos[n2])

    return distances


def get_auto_scale(G, pos, node_sizes, k=20):
    """ Returns the ratio of the ideal vs current top node distance.

    The scale is being set with a heuristic that we consider k largest nodes to prevent overlap,
    take the distances of all these nodes, find the current minimum top node distance. We also 
    find the ideal minimum top node distance that will prevent overlap and return their ratio to scale
    the layout by that scale.
    
    Parameters
    ----------

    G : nx.Graph

        A networkx Graph
    
    pos : dict

        A dictionary containing the node positions
    
    node_sizes : dict

        A dictionary containing the node sizes.
    
    k : int, default 20.

        Number of nodes to consider when setting the scale automatically.
    
    """
    result = _get_pairwise_distance_between_largest_nodes(
        G, pos, node_sizes, k=20)
    current_minimum_top_node_distance = min(result.values())
    nodes = min(result, key=result.get)
    ideal_minimum_top_node_distance = node_sizes[nodes[0]
                                                 ] + node_sizes[nodes[1]] + 50
    return ideal_minimum_top_node_distance/current_minimum_top_node_distance


def scale_layout(pos, scale):
    """ Scales layout
    
    Parameters
    ----------
    
    pos: dict
        A dictionary containing the positions of the nodes.
        
    scale :  int or float.
        
        Number to scale the positions by. 
        
    Returns
    -------
    dict

        Returns Dictionary containing Positions for the nodes scaled by the scale.
    
    """

    return {k: (v[0]*scale, v[1]*scale) for k, v in pos.items()}



from xml.etree.cElementTree import ElementTree
import matplotlib.colors as colors


def _convert_rgb_to_hex(color_code):
    """Converts dict containing rgb color code to hex color code. 

    Parameters
    ----------
    color_code : dict
        A dictionary containing rgb color code. Example : {'r':0,'g':0,'b':0}
    
    Returns
    -------
    str
        hex code of form '#XXXXXX' 

    """
    rgb_color_code = color_code['r'], color_code['g'], color_code['b']
    hex_code = colors.rgb2hex([(1.0*float(x))/255 for x in rgb_color_code])
    return hex_code


def parse_colors(path, hex=False):
    """ Parses the color attribute from .gexf file viz tag and returns the dict with the node colors.

    Default nx.read_gexf() does not read the attributes from the visualization namespace
    of .gexf files so this function is used to read the color codes. 
    
    To learn more see : https://gephi.org/gexf/format/viz.html 


    Parameters
    ----------
    path : str
        File or filename of the graph in .gexf format.
    hex : bool
        whether to return the color codes in hex format. if False rgb color codes are returned.

    Returns
    -------
    dict
        Returns dict containing the color codes. 
        
        Example : {'1':'#FFFFFF'} or {'1':{'r':0,'g':0,'b':0}}

    """
    tree = ElementTree(file=path)
    root = tree.getroot()
    nodes = [child for child in root.iter() if "nodes" in child.tag][0]
    color_codes = {}
    for node in nodes:
        id = node.attrib['id']
        color_code = [
            child for child in node if "color" in child.tag][0].attrib
        if hex:
            hex_code = _convert_rgb_to_hex(color_code)
            color_codes[id] = hex_code
        else:
            color_codes[id] = color_code

    return color_codes


def parse_size(path):
    """Parses the size attribute from .gexf file viz tag and returns the dict with the node sizes.

    Default nx.read_gexf() does not read attributes from the visualization namespace
    of .gexf files so we use this function to get the node sizes. 
    
    To learn more see : https://gephi.org/gexf/format/viz.html 

    Parameters
    ----------
    path : str
        File or filename of the graph in .gexf format.
    
    Returns
    -------
    dict
        Returns dict containing the node sizes. Example : {'1':52.0} 

    """
    tree = ElementTree(file=path)
    root = tree.getroot()
    nodes = [child for child in root.iter() if "nodes" in child.tag][0]
    sizes = {}
    for node in nodes:
        id = node.attrib['id']
        size = [child for child in node if "size" in child.tag][0].attrib
        sizes[id] = float(size['value'])
    return sizes


def parse_position(path):
    """Parses the position attribute from .gexf file viz tag and returns the dict with the node sizes.

    Default nx.read_gexf() does not read attributes from the visualization namespace
    of .gexf files so we use this function to get the node positions, possibly after using layout 
    algorithms in Gephi. 
    
    To learn more see : https://gephi.org/gexf/format/viz.html 

    Parameters
    ----------
    path : str
        File or filename of the graph in .gexf format.
    
    Returns
    -------
    dict
        Returns dict containing the node positions. Example : {'1':(2.0,3.0)} 


    """
    tree = ElementTree(file=path)
    root = tree.getroot()
    nodes = [child for child in root.iter() if "nodes" in child.tag][0]
    positions = {}
    for node in nodes:
        id = node.attrib['id']
        position = [
            child for child in node if "position" in child.tag][0].attrib
        positions[id] = (float(position['x']), float(position['y']))
    return positions



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(name='mediaviz',
      version='0.1.1',
      description='Visualize Networks With Force Atlas 2 Layout',
      long_description = long_description,
          long_description_content_type="text/markdown",
      url='https://github.com/Tahsin-Mayeesha/Mediaviz',
      author='Tahsin Mayeesha',
      author_email='tasmiah.tahsin@hotmail.com',
      license='MIT',
      packages=setuptools.find_packages(),
      classifiers=[
        'Development Status :: 5 - Production/Stable',

        'Intended Audience :: Science/Research',
        'Topic :: Scientific/Engineering :: Mathematics',

        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',

        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6'
    ],

    install_requires=[ "fa2l",
        "networkx<2.0.0",
        "numpy", "matplotlib",'python-louvain',"adjustText"
    ],

    setup_requires=[
        'pytest-runner'
    ],
    tests_require=[
        'pytest'
    ],
     
      zip_safe=False)



def set_node_color(G, color_by, colormap=None):
    """ Returns a list of colors for each node based on the provided colormap. 

    Parameters 
    ----------

    G : graph.
        A networkx graph.

    color_by : str
        Graph node attribute with finite discrete categorical values.
        See : https://networkx.github.io/documentation/networkx-1.10/tutorial/tutorial.html#adding-attributes-to-graphs-nodes-and-edges for learning about node attributes

    colormap : 
        Dict mapping values of color_by attribute to different colors. colors can be a single color like 'r' or
        hex code of form '#FFFFFF'

    Returns
    -------
    list
        A list containing the colors for each node.

    Example 
    -------

    >>> colormap = {'male':'b','female':'r'}
    >>> node_colors = set_node_color(G,"gender",colormap)
    ['b','b','r','b']
    """
    node_colors = [colormap[n[1][color_by]] for n in G.nodes(data=True)]
    return node_colors


def set_node_size(G, size_field, min_size, max_size):
    """ Returns a list containing values of the size_field attribute from Graph range 
        normalized between [min_size,max_size] interval.

    Parameters 
    ----------

    G : graph.
        A networkx graph.

    size_field : graph attribute
        graph node attribute containing numbers(integers or floats). 
        See : https://networkx.github.io/documentation/networkx-1.10/tutorial/tutorial.html#adding-attributes-to-graphs-nodes-and-edges for learning about graph attributes

    min_size : float
        minimum value for the new range
    max_size : float
        maximum value for the new range.

    Notes
    --------

    Mapping/scaling a number to a certain range like [0,1] or [-1,1]

    y = (x - min(d))*(max(n)-min(n))
        _____________________________  + min(n)
            (max(d)-min(d))

    min(d) = minimum value in the data

    max(d) = maximum value in the data

    min(n) = minimum value in the new range
    
    max(n) = maximum value in the new range
    input number : x , output : y

    Example 
    -------

    >>>size_field = 'inlink_count'
    >>>node_sizes = set_node_size(G,size_field= "inlink_count",min_size = 0.1, max_size=800)

    Returns
    -------
    list
        list of node sizes normalized to the range.

    """

    try:
        import numpy as np
    except ImportError:
        raise ImportError("Numpy required for calculation")
    vals = np.array([n[1][size_field] for n in G.nodes(data=True)])
    min_val = vals.min()
    max_val = vals.max()
    return (((vals - min_val)*(max_size-min_size))/(max_val-min_val)) + min_size


def filter_graph(G, filter_by=None, top=None):
    """Filters the graph by returning the subgraph for the top x nodes for the given filter_by field.
    
    To learn about subgraphs see : https://networkx.github.io/documentation/networkx-1.10/reference/generated/networkx.Graph.subgraph.html

    Parameters  
    ----------

    G : graph.
        A networkx graph.

    filter_by : graph node attribute
        Numerical node attribute to filter the graph. 

    top : int
        Number of top nodes to return. 
        E.g if the node attribute is node_size then the function returns top = 100 largest node subgraph.


    Example  
    -------

    >>> filter_field = "inlink_count"
    >>> filter_graph(G,filter_by=filter_field,top=k)

    Returns
    -------
    nx.Graph
        Subgraph after filtering by the node attribute.
    """
    if top <= len(G.nodes()):
        filter = {n[0]: n[1][filter_by] for n in G.nodes(data=True)}
        filter_nodes = sorted(
            filter.items(), key=lambda x: x[1], reverse=True)[0:top]
        return G.subgraph([n[0] for n in filter_nodes])
    else:
        raise ValueError(
            "Subgraph can't contain more nodes than original graph")


def set_node_label(G, label_field):
    """ Returns a dict mapping nodes to the labels. 
    
    Helper function for use in visualization functions like nx.draw_networkx_labels.

    Parameters 
    ----------

    G : graph.
        A networkx graph.

    label_field : node attribute
        Node attribute to be set as a the label for the node.

    Example
    -------

    >>> label_field = "label"
    >>> labels = set_node_label(G,label_field)

    Returns
    -------
    list
        List of node labels

    """
    node_labels = dict((n[0], n[1][label_field]) for n in G.nodes(data=True))
    return node_labels


def edgecolor_by_source(G, node_colors):
    """ Returns a list of colors to set as edge colors based on the source node for each edge.

    Parameters 
    ----------

    G : graph.
        A networkx graph.

    node_colors : list 
        List of node colors.

    Example  
    --------

    >>> colormap = {'male':'b','female':'r'}
    >>> node_colors = set_node_color(G,"gender",colormap)
    >>> edge_colors = edgecolor_by_source(G,node_colors)

    Returns
    -------
    list
        list of colors for each edge of the graph color set by the source node.

    """

    edge_colormap = []
    node_colormap = dict(zip(G.nodes(), node_colors))
    for edge in G.edges():
        edge_colormap.append(node_colormap[edge[0]])
    return edge_colormap


def get_subgraph_pos(G, pos):
    """ Returns the filtered positions for subgraph G. If subgraph = original graph then pos will be returned.

    Parameters 
    ----------

    G : nx.Graph
        A graph object.
    Pos : dict
        A dictionary with nodes as keys and positions as values.

    Example 
    -------

    >>> pos  = nx.spring_layout(G)
    >>> subgraph_nodes = ['1','2','3']
    >>> subgraph = G.subgraph(subgraph_nodes)
    >>> subgraph_positions = get_subgraph_pos(subgraph,pos)

    Returns
    -------
    dict 
        Assuming positions were generated earlier for a larger graph with some layout algorithm
        this functions returns the filtered positions by the subgraph. 

    """
    return {k: v for k, v in pos.items() if k in G.nodes()}


def draw_networkx_nodes_custom(G, pos, node_size, node_color='r', alpha=1, ax=None, **kwargs):
    """Draws networkx graph nodes with circles instead of using plt.scatter function like draw_networkx_nodes

    Parameters
    ----------

    G : nx.Graph
        A networkx graph object.
    pos : dict
        A dictionary with nodes as keys and positions as values.
    node_size : list
        A list containing node sizes for each node.
    node_color : str or list 
        A single color or list containing color values. Behavior same as draw_networkx_nodes
    alpha : float
        opacity (between 0-1) for setting transparency of the nodes
    ax : axis 


    """
    try:
        import matplotlib
        import matplotlib.pyplot as plt
    except ImportError:
        raise ImportError("Matplotlib required for draw()")
    except RuntimeError:
        print("Matplotlib unable to open display")
        raise

    x = [v[0] for v in pos.values()]
    y = [v[1] for v in pos.values()]

    patches = [plt.Circle((x, y), radius=s)
               for x, y, s in zip(x, y, node_size)]

    coll = matplotlib.collections.PatchCollection(
        patches, color=node_color, alpha=alpha, **kwargs)
    ax.add_collection(coll)

    # ax.margins(0.01)


# https://stackoverflow.com/questions/32444037/how-can-i-plot-many-thousands-of-circles-quickly
